"""
__init__

Module for working with thread pools.
When creating a pool (use ThreadPool class), pass
the full class name (e.g. WMCore.ThreadPool.ThreadSlave).

Classes used in the threadpool overwrite the call method
of the ThreadSlave.

"""
