#!/usr/bin/env python
"""
_ListByStateAndLocation_

Oracle implementation of ListByStateAndLocation

Created on Feb 21, 2013

@author: dballest
"""

from WMCore.WMBS.MySQL.Jobs.ListByStateAndLocation import ListByStateAndLocation as MySQLListByStateAndLocation

class ListByStateAndLocation(MySQLListByStateAndLocation):
    pass
