#!/usr/bin/env python
"""
_SetStatus_

Oracle implementation of DBSBufferFiles.SetStatus
"""




from WMComponent.DBS3Buffer.MySQL.DBSBufferFiles.SetStatus import SetStatus as MySQLSetStatus

class SetStatus(MySQLSetStatus):
    """
    _SetStatus_

    Oracle implementation of SetStatus
    """
    pass
