# -*- coding: utf-8 -*-
"""
_SetRunningJobSlotsForSite_

Oracle implementation of ResourceControl.SetRunningJobSlotsForSite

Created on Fri Jun 22 14:16:24 2012

@author: dballest
"""

from WMCore.ResourceControl.MySQL.SetRunningJobSlotsForSite import \
        SetRunningJobSlotsForSite as MySQLSetRunningJobSlotsForSite

class SetRunningJobSlotsForSite(MySQLSetRunningJobSlotsForSite):
    pass
